@@ -239,7 +239,7 @@ class Solver:
                     outstr = f"static double input_data_fn[{len(self.model.listOfDataFunctions) * ncells}] = "
                     outstr += "{"
                     coords = self.model.domain.coordinates()
-                    for ndf, data_fn in enumerate(self.model.listOfDataFunctions):
+                    for ndf, data_fn in enumerate(self.model.listOfDataFunctions.values()):
                         for i in range(ncells):
                             if i > 0 and ndf == 0:
                                 outstr += ','
@@ -262,27 +262,26 @@ class Solver:
             input_constants += outstr
 
         num_types = len(self.model.domain.listOfTypeIDs)
-        outstr = f"const int input_num_subdomain = {num_types};\n"
+        outstr = f"const int input_num_subdomain = {num_types-1};\n" # the backend ignores type_id=0 (a.k.a. unassigned)
         input_constants += outstr
 
-        outstr = f"const double input_subdomain_diffusion_matrix[{len(self.model.listOfSpecies) * num_types}] = "
-        outstr += "{"
+        outstr = f"const double input_subdomain_diffusion_matrix[{len(self.model.listOfSpecies) * (num_types-1)}] = "
+        outvec = []
         for i, species in enumerate(self.model.listOfSpecies.values()):
             for j, type_id in enumerate(self.model.domain.typeNdxMapping.keys()):
-                if i + j > 0:
-                    outstr += ','
+                if j==0: continue  # do not process type_id=0 in diffusion
                 try:
                     if species not in self.model.listOfDiffusionRestrictions or \
                        type_id in self.model.listOfDiffusionRestrictions[species]:
-                        outstr += f"{species.diffusion_coefficient}"
+                        outvec.append(f"{species.diffusion_coefficient}")
                     else:
-                        outstr += "0.0"
+                        outvec.append("0.0")
                 except KeyError as err:
                     print(f"error: {err}")
                     print(self.model.listOfDiffusionRestrictions)
                     raise SimulationError(f"error: {self.model.listOfDiffusionRestrictions}") from err
 
-        outstr += "};\n"
+        outstr += "{"+",".join(outvec)+"};\n"
         input_constants += outstr
 
         return input_constants
@@ -291,7 +290,7 @@ class Solver:
         output_step = "unsigned int get_next_output(ParticleSystem* system)\n{\n"
         output_step += "static int index = 0;\n"
         output_step += "const std::vector<unsigned int> output_steps = {"
-        output_step += f"{', '.join(self.model.output_steps.astype(str).tolist())}"
+        output_step += f"{', '.join(self.model.tspan.output_steps.astype(str).tolist())}"
         output_step += "};\nunsigned int next_step = output_steps[index];\n"
         output_step += "index++;\n"
         output_step += "return next_step;\n}\n"
@@ -312,9 +311,9 @@ class Solver:
     def __get_particle_inits(self, num_chem_species):
         init_particles = ""
         if self.model.domain.type_id is None:
-            self.model.domain.type_id = ["type 1"] * self.model.domain.get_num_voxels()
+            self.model.domain.type_id = ["type_1"] * self.model.domain.get_num_voxels()
         for i, type_id in enumerate(self.model.domain.type_id):
-            if type_id is None:
+            if "UnAssigned" in type_id:
                 errmsg = "Not all particles have been defined in a type. Mass and other properties must be defined"
                 raise SimulationError(errmsg)
             x = self.model.domain.coordinates()[i, 0]
@@ -376,7 +375,7 @@ class Solver:
                             num_stoch_species, num_stoch_rxns, num_data_fn):
         system_config = f"debug_flag = {self.debug_level};\n"
         system_config += "ParticleSystem *system = new ParticleSystem("
-        system_config += f"{num_types},{num_chem_species},{num_chem_rxns},"
+        system_config += f"{num_types-1},{num_chem_species},{num_chem_rxns},"
         system_config += f"{num_stoch_species},{num_stoch_rxns},{num_data_fn});\n"
         system_config += f"system->static_domain = {int(self.model.staticDomain)};\n"
         if len(self.model.listOfSpecies) > 0:
@@ -386,8 +385,8 @@ class Solver:
             system_config += "system->stoch_rxn_propensity_functions = ALLOC_propensities();\n"
             system_config += "system->species_names = input_species_names;\n"
 
-        system_config += f"system->dt = {self.model.timestep_size};\n"
-        system_config += f"system->nt = {self.model.num_timesteps};\n"
+        system_config += f"system->dt = {self.model.tspan.timestep_size};\n"
+        system_config += f"system->nt = {self.model.tspan.num_timesteps};\n"
         if self.h is None:
             self.h = self.model.domain.find_h()
         if self.h == 0.0:
